@@ -83,6 +83,8 @@ export interface CompletionSection {
 export class CompletionContext {
   /// @internal
   abortListeners: (() => void)[] | null = []
+  /// @internal
+  abortOnDocChange = false
 
   /// Create a new completion context. (Mostly useful for testing
   /// completion sources—in the editor, the extension will create
@@ -96,7 +98,13 @@ export class CompletionContext {
     /// implicitly by typing. The usual way to respond to this is to
     /// only return completions when either there is part of a
     /// completable entity before the cursor, or `explicit` is true.
-    readonly explicit: boolean
+    readonly explicit: boolean,
+    /// The editor view. May be undefined if the context was created
+    /// in a situation where there is no such view available, such as
+    /// in synchronous updates via
+    /// [`CompletionResult.update`](#autocomplete.CompletionResult.update)
+    /// or when called by test code.
+    readonly view?: EditorView
   ) {}
 
   /// Get the extent, content, and (if there is a token) type of the
@@ -126,8 +134,19 @@ export class CompletionContext {
   /// Allows you to register abort handlers, which will be called when
   /// the query is
   /// [aborted](#autocomplete.CompletionContext.aborted).
-  addEventListener(type: "abort", listener: () => void) {
-    if (type == "abort" && this.abortListeners) this.abortListeners.push(listener)
+  ///
+  /// By default, running queries will not be aborted for regular
+  /// typing or backspacing, on the assumption that they are likely to
+  /// return a result with a
+  /// [`validFor`](#autocomplete.CompletionResult.validFor) field that
+  /// allows the result to be used after all. Passing `onDocChange:
+  /// true` will cause this query to be aborted for any document
+  /// change.
+  addEventListener(type: "abort", listener: () => void, options?: {onDocChange: boolean}) {
+    if (type == "abort" && this.abortListeners) {
+      this.abortListeners.push(listener)
+      if (options && options.onDocChange) this.abortOnDocChange = true
+    }
   }
 }
 
